package ch.std.jumpstartreactive.client;

import java.net.URL;
import java.util.List;
import java.util.Map;

import org.springframework.http.MediaType;
import org.springframework.web.reactive.function.client.WebClient;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

public class ReactiveCityClient {

	private URL url;
	
	public ReactiveCityClient(URL url) {
		this.url = url;
	}

	public Flux<String> getCitiesAsync() {		
		return WebClient.create(this.url.toString()).get().uri("/cities")
				.accept(MediaType.TEXT_EVENT_STREAM)
				.retrieve().bodyToFlux(String.class);			
	}

	public List<String> getCitiesSync() {		
		return WebClient.create(this.url.toString()).get().uri("/cities")
				.accept(MediaType.TEXT_EVENT_STREAM)
				.retrieve().bodyToFlux(String.class).collectList().block();			
	}

	public Flux<String> getCitiesListAsync() {		
		return WebClient.create(this.url.toString()).get().uri("/citieslist")
				.accept(MediaType.APPLICATION_STREAM_JSON)
				.retrieve().bodyToFlux(String.class);			
	}

	public List<String> getCitiesListSync() {		
		return WebClient.create(this.url.toString()).get().uri("/citieslist")
				.accept(MediaType.APPLICATION_STREAM_JSON)
				.retrieve().bodyToFlux(String.class).collectList().block();			
	}

	public Flux<Map> getCitiesMapAsync() {		
		return WebClient.create(this.url.toString()).get().uri("/citiesmap")
				.accept(MediaType.APPLICATION_STREAM_JSON)
				.retrieve().bodyToFlux(Map.class);			
	}

	public List<Map> getCitiesMapSync() {		
		return WebClient.create(this.url.toString()).get().uri("/citiesmap")
				.accept(MediaType.APPLICATION_STREAM_JSON)
				.retrieve().bodyToFlux(Map.class).collectList().block();			
	}

	public Flux<Map> getCitiesMapJSONAsync() {		
		return WebClient.create(this.url.toString()).get().uri("/citiesmapjson")
				.accept(MediaType.APPLICATION_JSON)
		.retrieve().bodyToFlux(Map.class);			
	}

	public List<Map> getCitiesMapJSONSync() {		
		return WebClient.create(this.url.toString()).get().uri("/citiesmapjson")
				.accept(MediaType.APPLICATION_JSON)
		.retrieve().bodyToFlux(Map.class).collectList().block();			
	}

	public Mono<String> getCityAsync(String value) {		
		return WebClient.create(this.url.toString()).get().uri("/city?value=" + value).accept(MediaType.TEXT_EVENT_STREAM).retrieve().bodyToMono(String.class);
	}
	
	public String getCitySync(String value) {		
		return WebClient.create(this.url.toString()).get().uri("/city?value=" + value).accept(MediaType.TEXT_EVENT_STREAM).retrieve().bodyToMono(String.class).block();
	}

	public Mono<Map> getCityMapAsync(String value) {		
		return WebClient.create(this.url.toString()).get().uri("/citymap?value=" + value).accept(MediaType.APPLICATION_STREAM_JSON).retrieve().bodyToMono(Map.class);
	}
	
	public Map getCityMapSync(String value) {		
		return WebClient.create(this.url.toString()).get().uri("/citymap?value=" + value).accept(MediaType.APPLICATION_STREAM_JSON).retrieve().bodyToMono(Map.class).block();
	}

	public Mono<Map> getCityMapJSONAsync(String value) {		
		return WebClient.create(this.url.toString()).get().uri("/citymapjson?value=" + value).accept(MediaType.APPLICATION_JSON).retrieve().bodyToMono(Map.class);
	}

	public Map getCityMapJSONSync(String value) {		
		return WebClient.create(this.url.toString()).get().uri("/citymapjson?value=" + value).accept(MediaType.APPLICATION_JSON).retrieve().bodyToMono(Map.class).block();
	}

}
